/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.InterruptableConsumer;
import com.github.sviperll.Consumer;
import com.github.sviperll.Supplier;
import com.github.sviperll.io.Closeables;
import java.util.Iterator;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class CloseableStreamSupplierBehaviour<T> implements CloseableStreamBehaviour<T> {
    private final Supplier<? extends CloseableStream<T>> closeableStreamFactory;

    public CloseableStreamSupplierBehaviour(Supplier<? extends CloseableStream<T>> closeableStreamFactory) {
        this.closeableStreamFactory = closeableStreamFactory;
    }

    @Override
    public CloseableIterator<T> openIterator() {
        Owned<CloseableStream<T>> closeableStream = Owned.newInstance(closeableStreamFactory.get());
        try {
            Owned<CloseableIterator<T>> closeableIterator = Owned.newInstance(closeableStream.value().stream().openIterator());
            try {
                Iterator<T> iterator = closeableIterator.value();
                return new CloseableIterator<T>(iterator, Closeables.combine(closeableIterator.take(), closeableStream.take()));
            } finally {
                closeableIterator.close();
            }
        } finally {
            closeableStream.close();
        }
    }

    @Override
    public void forEach(InterruptableConsumer<? super T> consumer) {
        CloseableStream<T> closeableStream = closeableStreamFactory.get();
        try {
            closeableStream.stream().forEach(consumer);
        } finally {
            closeableStream.close();
        }
    }

}
