/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.InterruptableConsumer;
import com.github.sviperll.Consumer;
import java.util.Iterator;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class IterableStreamBehaviour<T> implements StreamBehaviour<T> {
    private final Iterable<T> iterable;

    public IterableStreamBehaviour(Iterable<T> iterable) {
        this.iterable = iterable;
    }

    @Override
    public Iterator<T> openIterator() {
        return iterable.iterator();
    }

    @Override
    public void forEach(InterruptableConsumer<? super T> consumer) {
        for (T element : iterable) {
            if (!consumer.needsMore())
                break;
            consumer.accept(element);
        }
    }

}
