/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import com.github.sviperll.InterruptableConsumer;
import com.github.sviperll.Consumer;
import com.github.sviperll.Function;
import java.util.Iterator;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class MappedStreamBehaviour<T, U> implements CloseableStreamBehaviour<U> {
    private final CloseableStreamBehaviour<T> upstream;
    private final Function<? super T, ? extends U> function;
    public MappedStreamBehaviour(CloseableStreamBehaviour<T> upstream, Function<? super T, ? extends U> function) {
        this.upstream = upstream;
        this.function = function;
    }

    @Override
    public CloseableIterator<U> openIterator() {
        Owned<CloseableIterator<T>> closeable = Owned.newInstance(upstream.openIterator());
        try {
            final Iterator<T> iterator = closeable.value();
            Iterator<U> mappedIterator = new Iterator<U>() {
                @Override
                public boolean hasNext() {
                    return iterator.hasNext();
                }

                @Override
                public U next() {
                    return function.apply(iterator.next());
                }

                @Override
                public void remove() {
                    iterator.remove();
                }
            };
            return new CloseableIterator<U>(mappedIterator, closeable.take());
        } finally {
            closeable.close();
        }
    }

    @Override
    public void forEach(final InterruptableConsumer<? super U> consumer) {
        upstream.forEach(new InterruptableConsumer<T>() {
            @Override
            public void accept(T value) {
                consumer.accept(function.apply(value));
            }

            @Override
            public boolean needsMore() {
                return consumer.needsMore();
            }
        });
    }
}
