/*
 * Copyright 2013 Victor Nazarov <asviraspossible@gmail.com>.
 */
package org.github.sviperll.stream;

import java.io.Closeable;
import java.io.IOException;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class Owned<T extends Closeable> implements Closeable {
    public static <T extends Closeable> Owned<T> newInstance(T resource) {
        return new Owned<T>(resource);
    }

    private T value;
    private boolean owns = true;
    private Owned(T value) {
        this.value = value;
    }

    public void set(T resource) {
        close();
        value = resource;
        owns = true;
    }

    public T value() {
        if (!owns)
            throw new IllegalStateException("Value is not owned by pointer");
        return value;
    }

    public T take() {
        T result = value();
        owns = false;
        value = null;
        return result;
    }

    @Override
    public void close() {
        if (owns) {
            try {
                value.close();
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }

}
